/*
 * Project name:
     nRFControlSlave
 * Copyright:
     (c) Mikroelektronika, 2013.
 * Description:
      A simple example of communication over nRF with Mikromedia + for PIC32MX7
 * Test configuration:
     MCU:             P32MX795F512L
                      http://ww1.microchip.com/downloads/en/DeviceDoc/61156F.pdf
     dev.boar         EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.0000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
 * NOTES:
     - Turn on PORTA LEDs on SW15. (board specific)

 */

#include "__Lib_nRF24L01.h"
#include "nRF24L01_defs.h"
#include "led_protocol.h"

sbit nRF_CS at RC2_bit;
sbit nRF_CS_Direction at TRISC2_bit;

sbit nRF_CE at RC1_bit;
sbit nRF_CE_Direction at TRISC1_bit;

sbit nRF_IRQ at RE8_bit;
sbit nRF_IRQ_Direction at TRISE8_bit;

TnRF_Status nRFStatus;
TRF_Config rfConfig;

unsigned char rfState;
//unsigned long readPort;

TLed leds[8];

//Initialization of Leds data
void InitLeds(void) {
  unsigned char i;

  for (i = 0; i < 8; i++)
  {
    leds[i]._state = 0;
    leds[i]._no    = i;
  }
}

//Initialization of Timer 2
void InitTimer2(){
  // Initialize TIMER2 module for CE pulse
  T2CON             = 0x8000;
  T2IE_bit          = 1;
  T2IF_bit          = 0;
  T2IP0_bit         = 0;
  T2IP1_bit         = 1;
  T2IP2_bit         = 1;
  PR2               = 1280;
  TMR2              = 0;
  ON__T2CON_bit     = 1;
}

//Interrupt function for Timer 2
void Timer2_interrupt() iv IVT_TIMER_2 ilevel 7 ics ICS_SRS {
  T2IF_bit = 0;
  TIMER2_IRQ();
  T2IE_bit = 0;                    // stop timer 2
}

//Interrupt function for Change Notice Interrupt
void INT1_Interrupt() iv IVT_EXTERNAL_1 ilevel 7 ics ICS_SRS {
  INT1IF_bit = 0;
  nRF24L01_IRQ();
}

//Initialization of External Interrupt
void InitExt(){
  INT1IP0_bit = 1;
  INT1IP1_bit = 1;
  INT1IP2_bit = 1;
  
  INTCONbits.INT1EP = 0;
  
  INT1IF_bit = 0; // clear external interrupt 1 flag
  INT1IE_bit = 1; // enable external interrupt 1
}

void InitRF()
{
  nRF_CE = 0;
  nRF_CS = 1;

  DisableInterrupts();
  //Initialize timer and external interrupt
  InitTimer2();
  InitExt();
  
  EnableInterrupts(); // enable all interrupts

  SPI3_Init();
}

void SetVal(unsigned char off, unsigned char *val)
{
  if(off == LED_OFF)
  {
    *val = 0;
  }
  else
  {
    *val = 1;
  }
}

//Command used to control all leds on port (On/Off state)
void CmdAll(unsigned char off)
{
  unsigned char val, i;
  val = 0;
  SetVal(off, &val);

  for(i = 0; i < 8; i++)
  {
    leds[i]._state = val;

    if(val == LED_ON)
    {
      LATA |= val << i;
    }
    else
    {
      LATA &= val << i;
    }
  }
}

//Command used to control leds in a column.
void CmdColumn(unsigned char off)
{
  unsigned char val, i;
  val = 0;
  SetVal(off, &val);

  if((RX_pload[0] & 0x0F) == LED_CLN_01)
  {
    if(val == 1)
      LATA |= (val << 4) | (val << 5) | (val << 6) | (val << 7);
    else
      LATA &= 0x0F;

    for(i = 4; i < 8; i++)
    {
      leds[i]._state = val;
    }
  }
  else
  {
    if(val == 1)
      LATA |= val | (val << 1) | (val << 2) | (val << 3);
    else
      LATA &= 0xF0;

    for(i = 0; i < 4; i++)
    {
      leds[i]._state = val;
    }
  }
}

//Command used to control all leds in a row.
void CmdRow(unsigned char off)
{
  unsigned char val, l1, l2, tmp;;
  val = 0;

  SetVal(off, &val);

  l1 = 7 - (RX_pload[0] & 0x0F);
  l2 = 3 - (RX_pload[0] & 0x0F);

  leds[l1]._state = val;
  leds[l2]._state = val;

  tmp = (1 << l1) | (1 << l2);

  if(val == 1)
    LATA |= (val << l1) | (val << l2);
  else
    LATA &= ~tmp;
}

//Command used to control a single Led.
void CmdSingleLed(unsigned char off)
{
  unsigned char val, tmp;

  val = 0;
  SetVal(off, &val);

  leds[((RX_pload[0] & 0x0F) - 1)]._state = val;

  tmp = (1 << ((RX_pload[0] & 0x0F) - 1));

  if(val == 0)
    LATA &= ~tmp;
  else
    LATA |= tmp;
}

void ProcessCmd(unsigned char *cmd)
{
  //Get command bits from RX buffer.
  switch((RX_pload[0] & 0xF0))
  {
    case LED_CMD_ALL_ON :
    {
      CmdAll(LED_ON); break;
    }

    case LED_CMD_SINGLE_ON:
    {
      CmdSingleLed(LED_ON);
      break;
    }

    case LED_CMD_ROW_ON:
    {
      CmdRow(LED_ON);
      break;
    }

    case LED_CMD_COLUMN_ON:
    {
      CmdColumn(LED_ON);
      break;
    }

    case LED_CMD_ALL_OFF:
    {
      CmdAll(LED_OFF); break;
    }

    case LED_CMD_SINGLE_OFF:
    {
      CmdSingleLed(LED_OFF);
      break;
    }

    case LED_CMD_ROW_OFF:
    {
      CmdRow(LED_OFF);
      break;
    }

    case LED_CMD_COLUMN_OFF:
    {
      CmdColumn(LED_OFF);
      break;
    }
  }
}

unsigned char tmp;

void main() {
  AD1PCFG = 0xFFFF;
  JTAGEN_bit = 0;
  TRISA = 0;
  LATA  = 0;

  InitLeds();
  
  //nRF Config Settings
  rfConfig.autoTransmitDelay = nRF_AUTO_RETRANSMIT_DELAY_4000us;
  rfConfig.autoTransmitCount = nRF_AUTO_RETRANSMIT_COUNT_05;
  rfConfig.channel           = 64;
  rfConfig.outputPower       = nRF_OUTPUT_POWER_0dBm;
  rfConfig.dataRate          = nRF_DATA_RATE_1Mbps;
  
  nRF_Init();
  InitRF();
  Delay_ms(100);
  
  rfState = RF_STOP;
  
  nRF_RX_Mode_Init_Advanced(rfConfig.dataRate, rfConfig.outputPower, rfConfig.channel);

  while (1) {
    if(nRF_Receive_Packet())
    {
      nRF_Process(&nRFStatus);
      if(nRFStatus._status == nRF_RX_DR)
      {
          ProcessCmd(RX_pload);
      }
    }
  }
}